#include <Wire.h>
#include <WiFi.h>
#include <time.h>
#include "RTClib.h"
#include "Adafruit_HTS221.h"
#include "Adafruit_Sensor.h"
#include <Adafruit_GFX.h>
#include <Adafruit_ST7735.h>
#include "ImageIO.h"
#include "digits.h"

#define TIMEZONE_JST (3600 * 9) // 日本標準時：UTC(世界標準時)より９時間早い
#define DAYLIGHTOFFSET_JST (0)  // サマータイムなし

const char* ntp_server1 = "ntp.nict.jp";
const char* ntp_server2 = "pool.ntp.org";

static char *ssid[] = {"xxxxxxxxxxxx","xxxxxxxxxxxx"};
static char *password[] = {"xxxxxxxxxxxx","xxxxxxxxxxxx"};
int  wifi_cnt = 2;
static time_t t = 0;

RTC_PCF8523 rtc;

#define TFT_CS    A2    //  9
#define TFT_DC    A1    // 17
#define TFT_SCLK  SCK   // 36
#define TFT_MOSI  MOSI  // 35
#define TFT_RST   -1    // 35
#define TFT_LIT   A3    //  8
#define BTN_PIN   A0    // 18

#define MODE_TEMPERATURE  1
#define MODE_HUMIDITY     2
#define MODE_CLOCK        3
#define KEEP_TIME     10000

uint8_t mode = MODE_TEMPERATURE;
float   offset_temperature = -2.25;

Adafruit_HTS221 hts;
Adafruit_ST7735 tft = Adafruit_ST7735(TFT_CS, TFT_DC, TFT_MOSI,TFT_SCLK, TFT_RST);

ImageIO       imgio;
unsigned char *digits[10];
unsigned char *colon;
uint8_t       digit_width  = 32;
uint8_t       digit_height = 48;
uint8_t       colon_width  =  7;
uint8_t       colon_height = 32;
uint8_t       last[5];

typedef struct {
	unsigned char  R;
	unsigned char  G;
	unsigned char  B;
} BMP_RGB;
BMP_RGB *RGB;

void get_ntp(int num) {

	int cnt = 0;

	WiFi.begin(ssid[num], password[num]);
	while (WiFi.status() != WL_CONNECTED) {
		if ((++cnt)==20) return;
		delay(500);
	}

	configTime(TIMEZONE_JST, DAYLIGHTOFFSET_JST, ntp_server1, ntp_server2);

	for (cnt=0;cnt<10;++cnt) {
		t = time(NULL);
		if (t>TIMEZONE_JST) break;
		delay(500);
	}
	WiFi.disconnect();
}

void show_temperature()
{
	char    temperature[6];

	sensors_event_t se_temp;
	sensors_event_t se_humidity;
	hts.getEvent(&se_humidity, &se_temp);// populate temp and humidity objects with fresh data
	sprintf(temperature,"%05.2f",(se_temp.temperature + offset_temperature));
	temperature[2]=':';

	if (last[0]!=temperature[0]) {
		last[0] = temperature[0];
		RGB = (BMP_RGB*)digits[last[0]-0x30];
		for(int y=0; y < digit_height; y++) {
			for(int x=0; x < digit_width; x++, RGB++) {
				if (last[0]=='0') {
					tft.drawPixel(x+10,y+16,0x0000); // black
				} else {
					tft.drawPixel(x+10,y+16,tft.color565(RGB->R,RGB->G,RGB->B));
				}
			}
		}
	}

	if (last[1]!=temperature[1]) {
		last[1] = temperature[1];
		RGB = (BMP_RGB*)digits[last[1]-0x30];
		for(int y=0; y < digit_height; y++) {
			for(int x=0; x < digit_width; x++, RGB++) {
				tft.drawPixel(x+42,y+16,tft.color565(RGB->R,RGB->G,RGB->B));
			}
		}
	}

	if (last[2]!=temperature[2]) {
		last[2] = temperature[2];
		RGB = (BMP_RGB*)colon;
		for(int y=0; y < colon_height; y++) {
			for(int x=0; x < colon_width; x++, RGB++) {
				tft.drawPixel(x+76,y+24,tft.color565(RGB->R,RGB->G,RGB->B)); // orange
			}
		}
	}

	if (last[3]!=temperature[3]) {
		last[3] = temperature[3];
		RGB = (BMP_RGB*)digits[last[3]-0x30];
		for(int y=0; y < digit_height; y++) {
			for(int x=0; x < digit_width; x++, RGB++) {
				tft.drawPixel(x+85,y+16,tft.color565(RGB->R,RGB->G,RGB->B));
			}
		}
	}

	if (last[4]!=temperature[4]) {
		last[4] = temperature[4];
		RGB = (BMP_RGB*)digits[last[4]-0x30];
		for(int y=0; y < digit_height; y++) {
			for(int x=0; x < digit_width; x++, RGB++) {
				tft.drawPixel(x+117,y+16,tft.color565(RGB->R,RGB->G,RGB->B));
			}
		}
	}

	loop_delay();
}

void show_humidity()
{
	char humidity[6];

	sensors_event_t se_temp;
	sensors_event_t se_humidity;
	hts.getEvent(&se_humidity, &se_temp);// populate temp and humidity objects with fresh data
	sprintf((char*)humidity,"%05.2f",se_humidity.relative_humidity);
	humidity[2]=':';

	if (last[0]!=humidity[0]) {
		last[0] = humidity[0];
		RGB = (BMP_RGB*)digits[last[0]-0x30];
		for(int y=0; y < digit_height; y++) {
			for(int x=0; x < digit_width; x++, RGB++) {
				if (last[0]=='0') {
					tft.drawPixel(x+10,y+16,0x0000); // black
				} else {
					tft.drawPixel(x+10,y+16,tft.color565(RGB->B,RGB->G,RGB->R));
				}
			}
		}
	}

	if (last[1]!=humidity[1]) {
		last[1] = humidity[1];
		RGB = (BMP_RGB*)digits[last[1]-0x30];
		for(int y=0; y < digit_height; y++) {
			for(int x=0; x < digit_width; x++, RGB++) {
				tft.drawPixel(x+42,y+16,tft.color565(RGB->B,RGB->G,RGB->R));
			}
		}
	}

	if (last[2]!=humidity[2]) {
		last[2] = humidity[2];
		RGB = (BMP_RGB*)colon;
		for(int y=0; y < colon_height; y++) {
			for(int x=0; x < colon_width; x++, RGB++) {
				tft.drawPixel(x+76,y+24,tft.color565(RGB->B,RGB->G,RGB->R)); // cyan
			}
		}
	}

	if (last[3]!=humidity[3]) {
		last[3] = humidity[3];
		RGB = (BMP_RGB*)digits[last[3]-0x30];
		for(int y=0; y < digit_height; y++) {
			for(int x=0; x < digit_width; x++, RGB++) {
				tft.drawPixel(x+85,y+16,tft.color565(RGB->B,RGB->G,RGB->R));
			}
		}
	}

	if (last[4]!=humidity[4]) {
		last[4] = humidity[4];
		RGB = (BMP_RGB*)digits[last[4]-0x30];
		for(int y=0; y < digit_height; y++) {
			for(int x=0; x < digit_width; x++, RGB++) {
				tft.drawPixel(x+117,y+16,tft.color565(RGB->B,RGB->G,RGB->R));
			}
		}
	}

	loop_delay();
}

void show_clock()
{
	char clock[6];

	DateTime now = rtc.now();
	sprintf((char*)clock,"%02d:%02d",now.hour(),now.minute());

	if (last[0]!=clock[0]) {
		last[0] = clock[0];
		RGB = (BMP_RGB*)digits[last[0]-0x30];
		for(int y=0; y < digit_height; y++) {
			for(int x=0; x < digit_width; x++, RGB++) {
				if (last[0]=='0') {
					tft.drawPixel(x+10,y+16,0x0000); // black
				} else {
					tft.drawPixel(x+10,y+16,tft.color565(RGB->B,RGB->R,RGB->G));
				}
			}
		}
	}

	if (last[1]!=clock[1]) {
		last[1] = clock[1];
		RGB = (BMP_RGB*)digits[last[1]-0x30];
		for(int y=0; y < digit_height; y++) {
			for(int x=0; x < digit_width; x++, RGB++) {
				tft.drawPixel(x+42,y+16,tft.color565(RGB->B,RGB->R,RGB->G));
			}
		}
	}

	if (last[2]!=clock[2]) {
		last[2] = clock[2];
		RGB = (BMP_RGB*)colon;
		for(int y=0; y < colon_height; y++) {
			for(int x=0; x < colon_width; x++, RGB++) {
				tft.drawPixel(x+76,y+24,tft.color565(RGB->B,RGB->R,RGB->G)); // cyan
			}
		}
	}

	if (last[3]!=clock[3]) {
		last[3] = clock[3];
		RGB = (BMP_RGB*)digits[last[3]-0x30];
		for(int y=0; y < digit_height; y++) {
			for(int x=0; x < digit_width; x++, RGB++) {
				tft.drawPixel(x+85,y+16,tft.color565(RGB->B,RGB->R,RGB->G));
			}
		}
	}

	if (last[4]!=clock[4]) {
		last[4] = clock[4];
		RGB = (BMP_RGB*)digits[last[4]-0x30];
		for(int y=0; y < digit_height; y++) {
			for(int x=0; x < digit_width; x++, RGB++) {
				tft.drawPixel(x+117,y+16,tft.color565(RGB->B,RGB->R,RGB->G));
			}
		}
	}

	loop_delay();
}

void loop_delay()
{
	for (int interval=0; interval<KEEP_TIME; interval+=100) {
		if ( digitalRead(BTN_PIN)==LOW ) {
			delay(100);
			if (++mode > MODE_CLOCK) mode = MODE_TEMPERATURE;
			memset(last,0x0,5);
			break;
		} else {
			delay(100);
		}
	}
}

void setup(void)
{
	uint8_t  *RGBs;
	uint32_t digitSize, iconSize,colonSize;

  // the delay function does not disable interrupts. 
	pinMode(BTN_PIN, INPUT_PULLUP);

	if (! rtc.begin()) {
		while (1) delay(10);
	}

	for (int i=0;i<wifi_cnt;++i) {
		get_ntp(i);
		if (t>TIMEZONE_JST) {
			struct tm* tm = localtime(&t);
			rtc.adjust(DateTime(tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday, tm->tm_hour, tm->tm_min, tm->tm_sec));
			break;
		}
	}
	rtc.start();

	//channel,周波数,解像度(8bit=256)
	ledcSetup(0,12800,8); 
	//ピンをチャンネルに接続
	ledcAttachPin(TFT_LIT,0);
	ledcWrite(0,32);

	tft.initR(INITR_MINI160x80);
	tft.setTextWrap(false);
	tft.setRotation(3);
	tft.fillScreen(ST77XX_BLACK);

	// Try to initialize!
	if (!hts.begin_I2C()) {
		while (1) { delay(10); }
	}

	digitSize  = digit_width * digit_height * 3;
	iconSize   = 24 * 24 * 3;
	colonSize  =  7 * 32 * 3;
	for (uint8_t i=0;i<10;i++) digits[i] = (unsigned char*)malloc(digitSize);
	colon = (unsigned char*)malloc(colonSize);

	RGBs = imgio.Bmp2Rgb(d0_data, d0_len);
	memcpy(digits[0],RGBs,digitSize);
	RGBs = imgio.Bmp2Rgb(d1_data, d1_len);
	memcpy(digits[1],RGBs,digitSize);
	RGBs = imgio.Bmp2Rgb(d2_data, d2_len);
	memcpy(digits[2],RGBs,digitSize);
	RGBs = imgio.Bmp2Rgb(d3_data, d3_len);
	memcpy(digits[3],RGBs,digitSize);
	RGBs = imgio.Bmp2Rgb(d4_data, d4_len);
	memcpy(digits[4],RGBs,digitSize);
	RGBs = imgio.Bmp2Rgb(d5_data, d5_len);
	memcpy(digits[5],RGBs,digitSize);
	RGBs = imgio.Bmp2Rgb(d6_data, d6_len);
	memcpy(digits[6],RGBs,digitSize);
	RGBs = imgio.Bmp2Rgb(d7_data, d7_len);
	memcpy(digits[7],RGBs,digitSize);
	RGBs = imgio.Bmp2Rgb(d8_data, d8_len);
	memcpy(digits[8],RGBs,digitSize);
	RGBs = imgio.Bmp2Rgb(d9_data, d9_len);
	memcpy(digits[9],RGBs,digitSize);

	RGBs = imgio.Bmp2Rgb(colon_data, colon_len);
	memcpy(colon,RGBs,colonSize);

	memset(last,0x0,5);

	setCpuFrequencyMhz(40);
}

void loop()
{
	switch (mode) {
		case MODE_TEMPERATURE:
			show_temperature();
			break;
		case MODE_HUMIDITY:
			show_humidity();
			break;
		case MODE_CLOCK:
			show_clock();
			break;
	}
}
